/*
 * Copyright Bigeon Emmanuel (2014)
 *
 * emmanuel@bigeon.fr
 *
 * This software is a computer program whose purpose is to
 * provide a generic framework for console applications.
 *
 * This software is governed by the CeCILL license under French law and
 * abiding by the rules of distribution of free software.  You can  use,
 * modify and/or redistribute the software under the terms of the CeCILL
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info".
 *
 * As a counterpart to the access to the source code and  rights to copy,
 * modify and redistribute granted by the license, users are provided only
 * with a limited warranty  and the software's author,  the holder of the
 * economic rights,  and the successive licensors  have only  limited
 * liability.
 *
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or
 * data to be ensured and,  more generally, to use and operate it in the
 * same conditions as regards security.
 *
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL license and that you accept its terms.
 */
/**
 * gclc:fr.bigeon.gclc.ConsoleApplicationTest.java
 * Created on: Jun 9, 2016
 */
package fr.bigeon.gclc;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertNull;
import static org.junit.Assert.assertTrue;

import java.io.IOException;

import org.junit.Test;

import fr.bigeon.gclc.command.ExitCommand;
import fr.bigeon.gclc.command.ICommand;
import fr.bigeon.gclc.exception.CommandRunException;
import fr.bigeon.gclc.exception.CommandRunExceptionType;
import fr.bigeon.gclc.exception.InvalidCommandName;
import fr.bigeon.gclc.i18n.Messages;
import fr.bigeon.gclc.manager.ConsoleManager;
import fr.bigeon.gclc.manager.SystemConsoleManager;
import fr.bigeon.gclc.manager.PipedConsoleManager;

/** Test class for ConsoleApplication
 *
 * @author Emmanuel Bigeon */
@SuppressWarnings({"javadoc", "nls", "static-method"})
public class ConsoleApplicationTest {

    /** 3 seconds in milliseconds */
    protected static final long THREE_SECONDS = 3000;

    /** Test the base of a console application */
    @Test
    public void test() {
        ConsoleTestApplication app = new ConsoleTestApplication(
                new SystemConsoleManager());
        app.exit();
    }

    @Test
    public void executionTest() {
        try (CommandTestingApplication application = new CommandTestingApplication()) {

            // remove welcome
            assertEquals(application.getApplication().getHeader(),
                    application.readNextLine());
            // Remove first prompt
            assertNull(application.readNextLine());
            application.sendCommand("");
            assertNull(application.readNextLine());
            application.sendCommand("test");
            assertEquals("Test command ran fine", application.readNextLine());
            assertNull(application.readNextLine());
            application.sendCommand("toto");
            assertEquals(
                    Messages.getString("ConsoleApplication.cmd.failed", "toto"),
                    application.readNextLine());
            assertEquals(
                    Messages.getString("CommandProvider.unrecognized", "toto"),
                    application.readNextLine());
            assertNull(application.readNextLine());
            application.sendCommand("long");
            assertEquals("Waita minute", application.readNextLine());
            assertEquals("done!", application.readNextLine());

            CommandRequestListener crl = new CommandRequestListener() {

                @Override
                public void commandRequest(String command) {
                    //
                }
            };
            CommandRequestListener crl2 = new CommandRequestListener() {

                @Override
                public void commandRequest(String command) {
                    //
                }
            };
            application.getApplication().addListener(crl);
            application.getApplication().addListener(crl2);
            assertNull(application.readNextLine());
            application.sendCommand("test");
            assertEquals("Test command ran fine", application.readNextLine());
            application.getApplication().removeListener(crl2);
            application.getApplication().removeListener(crl);
            application.getApplication().removeListener(crl);

            assertTrue(application.getApplication().isRunning());

            assertNull(application.readNextLine());
            application.sendCommand("exit");
            assertEquals(application.getApplication().getFooter(),
                    application.readNextLine());
            assertFalse(application.getApplication().isRunning());
        } catch (IOException e1) {
            assertNull(e1);
        }

        ConsoleApplication appli = null;
        try (PipedConsoleManager manager = new PipedConsoleManager()) {
            final ConsoleApplication app = new ConsoleApplication(manager, null,
                    null);
            appli = app;
            app.add(new ExitCommand("exit", app));

            Thread th = new Thread(new Runnable() {

                @SuppressWarnings("synthetic-access")
                @Override
                public void run() {
                    app.start();
                }
            });

            th.start();

            manager.type("exit");
            th.join();

        } catch (IOException | InvalidCommandName | InterruptedException e) {
            assertNull(e);
        }

        assertNotNull(appli);
        appli.start();
        assertFalse(appli.isRunning());
    }

    @Test
    public void interpretCommandTest() {
        try (PipedConsoleManager test = new PipedConsoleManager()) {
            ConsoleApplication appl = new ConsoleApplication(test, "", "");

            appl.interpretCommand("invalid cmd \"due to misplaced\"quote");
            assertEquals("Command line cannot be parsed", test.readNextLine());

            appl.interpretCommand("");

            final String message = "message";
            try {
                appl.add(new ICommand() {

                    @Override
                    public void execute(String... args) throws CommandRunException {
                        throw new CommandRunException(
                                CommandRunExceptionType.USAGE, message, this);
                    }

                    @Override
                    public String getCommandName() {
                        return "fail";
                    }

                    @Override
                    public void help(ConsoleManager manager,
                                     String... args) throws IOException {
                        manager.println(message);
                    }

                    @Override
                    public String tip() {
                        return "";
                    }

                });
            } catch (InvalidCommandName e) {
                assertNull(e);
            }
            appl.interpretCommand("fail");
            assertEquals(
                    Messages.getString("ConsoleApplication.cmd.failed", "fail"),
                    test.readNextLine());
            assertEquals(message, test.readNextLine());
            assertEquals(message, test.readNextLine());

        } catch (IOException e) {
            assertNull(e);
        }
    }
}
